pub use libaoc_macros::*;
pub use miette;
use miette::Context;
use owo_colors::OwoColorize;

#[macro_export]
macro_rules! run_days {
    ($($day:ident),+ $(,)?) => {
        libaoc::AocRunner::new()
            $(.day($day::aoc_day()))*
            .run()
    };
}

pub struct Day {
    pub name: String,
    pub part1: fn() -> miette::Result<()>,
    pub part2: Option<fn() -> miette::Result<()>>,
}

#[derive(Default)]
pub struct AocRunner {
    days: Vec<Day>,
}

impl AocRunner {
    pub fn new() -> Self {
        Self::default()
    }

    pub fn day(&mut self, day: Day) -> &mut Self {
        self.days.push(day);
        self
    }

    pub fn run(&self) -> miette::Result<()> {
        for day in &self.days {
            println!("Running day {}...", day.name.blue());

            println!("{}", "--- Part 1 ---".red());
            (day.part1)().wrap_err_with(|| format!("Running part 1 of day {}", &day.name))?;
            println!();

            if let Some(p2) = day.part2 {
                println!("{}", "--- Part 2 ---".red());
                p2().wrap_err_with(|| format!("Running part 2 of day {}", &day.name))?;
                println!();
            }
        }

        Ok(())
    }
}
